<?php
/* Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace Cybersponse\CoreBundle\Listener;


use Doctrine\DBAL\DBALException;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
use Symfony\Component\HttpKernel\EventListener\ExceptionListener;
use Symfony\Component\HttpKernel\Exception\ConflictHttpException;

class UniqueViolationExceptionListener extends ExceptionListener
{
    const UNIQUE_VIOLATION = 23505;
    const UNIQUE_VIOLATION_REGEX = "/Key \((.*?)\)=\((.*)\) already exists\./";

    public function onKernelException(GetResponseForExceptionEvent $event)
    {
        $exception = $event->getException();

        if ($exception instanceof DBALException) {
            $previous = $exception->getPrevious();
            if ($previous->getCode() == self::UNIQUE_VIOLATION) {
                preg_match(self::UNIQUE_VIOLATION_REGEX, $previous->getMessage(), $matches);

                $confictMessage = 'Unique violation: ';

                if (count($matches) == 3) {
                    $confictMessage = sprintf("Failed to create the record. The system administrator has defined a uniqueness constraint for the fields (%s) and a record already exists with the specified values for these fields!", $matches[1]);
                }else {
                    $explodedString = explode("Unique violation: 7 ERROR:", $previous->getMessage());
                    if (count($explodedString)) {
                        $confictMessage = trim($explodedString[1]);
                    }
                }
                $event->setException(new ConflictHttpException($confictMessage, $exception));
                parent::onKernelException($event);
            }
        }
    }
}
